
#include "LoRa/Radio/radio.h"

RadioEvents_t RadioEvents;

#define LORA_BANDWIDTH          0  // 0: 125 kHz, 1: 250 kHz, 2: 500 kHz, 3: Reserved
#define LORA_DATARATE           7  // 6: 64, 7: 128, 8: 256, 9: 512, 10: 1024, 11: 2048, 12: 4096  chips
#define LORA_CODINGRATE         1  // 1: 4/5, 2: 4/6, 3: 4/7, 4: 4/8
#define LORA_PREAMBLE_LENGTH    8  // Length in symbols (the hardware adds 4 more symbols)
#define LORA_SYMBOL_TIMEOUT     5  // Sets the RxSingle timeout in symbols
#define LORA_FIX_LENGTH_PAYLOAD 0  // Fixed length packets [0: variable, 1: fixed]
#define LORA_LENGTH_PAYLOAD     0  // For fixed length packets 
#define LORA_CRC_ENABLED        1  // 0: OFF, 1: ON
#define LORA_FHSS_ENABLED       0  // 0: OFF, 1: ON
#define LORA_NB_SYMB_HOP        4  // Number of symbols between each hop
#define LORA_IQ_INVERSION_ON    0  // Inverts IQ signals [0: not inverted, 1: inverted]
#define LORA_RX_CONTINUOUS      true //Sets the reception in continuous mode [false: single mode, true: continuous mode]

#define RF_FREQUENCY            868100000 // CH1

void setup( void )
{
    pinMode(PIN_LED_BLUE, OUTPUT);
    pinMode(PIN_LED_RED, OUTPUT);

    Serial.begin(9600);
    Serial.println("LoRa Rx Mode");
    Serial.println();

    RadioEvents.TxDone    = OnTxDone;
    RadioEvents.RxDone    = OnRxDone;
    RadioEvents.RxError   = OnRxError;
    RadioEvents.TxTimeout = OnTxTimeout;
    RadioEvents.RxTimeout = OnRxTimeout;    
    
    Radio.Init(&RadioEvents);
    Radio.SetRxConfig( MODEM_LORA, LORA_BANDWIDTH, LORA_DATARATE,
                         LORA_CODINGRATE, 0, LORA_PREAMBLE_LENGTH,
                         LORA_SYMBOL_TIMEOUT, LORA_FIX_LENGTH_PAYLOAD, LORA_LENGTH_PAYLOAD,
                         LORA_CRC_ENABLED, LORA_FHSS_ENABLED, LORA_NB_SYMB_HOP, 
                         LORA_IQ_INVERSION_ON, LORA_RX_CONTINUOUS );
    
    Radio.SetChannel(RF_FREQUENCY);
                         
    Serial.println(Radio.GetStatus());
    
    Radio.Rx(500);
}

#define   BUFFER_SIZE         32       
uint8_t   Buffer[BUFFER_SIZE];
uint16_t  BufferSize = 0;
int16_t   rssi;
int8_t    snr;

int i = 0;
bool RxMsg = false;
void loop( void )
{
    Radio.Rx(3500);
    digitalWrite(PIN_LED_BLUE, !digitalRead(PIN_LED_BLUE));
    
    if(RxMsg == true) {
        digitalWrite(PIN_LED_RED, HIGH);

        Serial.print("RX Length: ");
        Serial.println(BufferSize);
        Serial.print("RX RSSI: ");
        Serial.println(rssi);
        Serial.print("RX SNR: ");
        Serial.println(snr);
        Serial.println("");
        
        int i;
        for(i=0; i<BufferSize; i++)
        {
          Serial.print(Buffer[i], HEX);
          Serial.print(" ");
        }
        Serial.println("");
        delay(100);
        digitalWrite(PIN_LED_RED, LOW);
        RxMsg = false;
    }
    delay(500);
}

void OnTxDone()
{
    Radio.Sleep();
    Serial.println("OnTxDone");
}

void OnRxDone(uint8_t *payload, uint16_t len, int16_t r, int8_t s)
{
    Radio.Sleep( );
    Serial.println("OnRxDone");
    
    memcpy(Buffer, payload, len);
    BufferSize = len;
    rssi = r;
    snr  = s;
    RxMsg = true;
}

void OnTxTimeout()
{
    Radio.Sleep();
    Serial.println("OnTxTimeout");
}

void OnRxTimeout()
{
    Radio.Sleep();
    Serial.println("OnRxTimeout");
}

void OnRxError()
{
    Radio.Sleep();
    Serial.println("OnRxError");
}

