
#include "LoRa/Radio/radio.h"

RadioEvents_t RadioEvents;

#define TX_OUTPUT_POWER        -1  // -1 to 14 dBm
#define LORA_BANDWIDTH          0  // 0: 125 kHz, 1: 250 kHz, 2: 500 kHz, 3: Reserved
#define LORA_DATARATE           7  // 6: 64, 7: 128, 8: 256, 9: 512, 10: 1024, 11: 2048, 12: 4096  chips
#define LORA_CODINGRATE         1  // 1: 4/5, 2: 4/6, 3: 4/7, 4: 4/8
#define LORA_PREAMBLE_LENGTH    8  // Length in symbols (the hardware adds 4 more symbols)
#define LORA_FIX_LENGTH_PAYLOAD 0  // Fixed length packets [0: variable, 1: fixed]
#define LORA_CRC_ENABLED        1  // 0: OFF, 1: ON
#define LORA_FHSS_ENABLED       0  // 0: OFF, 1: ON
#define LORA_NB_SYMB_HOP        4  // Number of symbols between each hop
#define LORA_IQ_INVERSION_ON    0  // Inverts IQ signals [0: not inverted, 1: inverted]
#define LORA_TIMEOUT            2000

#define RF_FREQUENCY            868100000 // CH1

void setup( void )
{
    pinMode(PIN_LED_GREEN, OUTPUT);
    digitalWrite(PIN_LED_GREEN, HIGH);
    delay(100);
    digitalWrite(PIN_LED_GREEN, LOW);
    delay(100);
    digitalWrite(PIN_LED_GREEN, HIGH);
    delay(100);
    digitalWrite(PIN_LED_GREEN, LOW);
    delay(100);
        
    Serial.begin(9600);
    Serial.println("LoRa Tx Mode");
    Serial.println();

    RadioEvents.TxDone    = OnTxDone;
    RadioEvents.RxDone    = OnRxDone;
    RadioEvents.RxError   = OnRxError;
    RadioEvents.TxTimeout = OnTxTimeout;
    RadioEvents.RxTimeout = OnRxTimeout; 
       
    Radio.Init(&RadioEvents);
    Radio.SetTxConfig( MODEM_LORA, TX_OUTPUT_POWER, 0, LORA_BANDWIDTH,
                         LORA_DATARATE, LORA_CODINGRATE,
                         LORA_PREAMBLE_LENGTH, LORA_FIX_LENGTH_PAYLOAD,
                         LORA_CRC_ENABLED, LORA_FHSS_ENABLED, LORA_NB_SYMB_HOP, 
                         LORA_IQ_INVERSION_ON, LORA_TIMEOUT);
    
    Radio.SetChannel(RF_FREQUENCY);
                         
    Serial.println(Radio.GetStatus());

}

#define BUFFER_SIZE         32       
const uint8_t  SendMsg[]  = "ThisIsATest";
uint16_t       BufferSize = BUFFER_SIZE;
uint8_t        Buffer[BUFFER_SIZE];

int i = 0;

void loop( void )
{
    digitalWrite(PIN_LED_GREEN, LOW);
    memcpy(Buffer, SendMsg, sizeof(SendMsg));
    delay(10); 
    Radio.Send( Buffer, BufferSize );

    
    int i;
    for(i=0; i<BufferSize; i++)
    {
      Serial.print(Buffer[i], HEX);
      Serial.print(" ");
    }
    Serial.println();
    Serial.println();
    
    delay( 100);
    digitalWrite(PIN_LED_GREEN, HIGH);
    delay(5000);
    
}


void OnTxDone()
{
    Radio.Sleep();
    Serial.println("OnTxDone");
}

void OnRxDone(uint8_t *payload, uint16_t len, int16_t rssi, int8_t snr)
{
    Radio.Sleep( );
    digitalWrite(PIN_LED_GREEN, HIGH);
    
    Serial.print("RX Length: ");
    Serial.println(len);
    Serial.print("RX RSSI: ");
    Serial.println(rssi);
    Serial.print("RX SNR: ");
    Serial.println(snr);
    Serial.println("");
    
    int i;
    for(i=0; i<len; i++)
    {
      Serial.print(payload[i], HEX);
      Serial.print(" ");
    }
    Serial.println("");
    delay(100);
    digitalWrite(PIN_LED_GREEN, LOW);
}

void OnTxTimeout()
{
    Radio.Sleep();
    Serial.println("OnTxTimeout");
}

void OnRxTimeout()
{
    Radio.Sleep();
    Serial.println("OnRxTimeout");
}

void OnRxError()
{
    Radio.Sleep();
    Serial.println("OnRxError");
}
