
#include "LIS3DE.h"
#include "Adafruit_MPL115A2.h"
#include "PocketQube.h"

void blink(int);
void pocketAdc();
void pocketAcc();
void pocketPressure();
void pocketLora();
void pocketLoraInit();

Adafruit_MPL115A2 mpl115a2;


void setup() {
  // put your setup code here, to run once:
  PocketQube.init();
  
  Serial.begin(9600);

  PocketQube.redOn();
  PocketQube.greenOn();
  PocketQube.blueOn();
  
  while(PocketQube.button()==LOW);  
  Serial.println("ISTsat-1 - Poket Qube 1");

  Serial.print("\r\nPIN_BUTTON: ");
  if(PocketQube.button()==HIGH)
    Serial.println("Ok");
  else
    Serial.println("Fail");
    


  pocketLoraInit();
}

int i = 0;

void loop(){

  PocketQube.redOff();
  PocketQube.greenOff();
  PocketQube.blueOff();
  
  Serial.println("\r\nLeds Test:");
  Serial.println("   Red");
  blink(PIN_LED_RED  );
  Serial.println("   Green");
  blink(PIN_LED_GREEN);
  Serial.println("   Blue");
  blink(PIN_LED_BLUE );
  
  Serial.println();
  Serial.print("Iteration: ");
  Serial.println(i++);

  Serial.print("MCU temperature: ");
  Serial.println(PocketQube.getTemperature());
  
  pocketAdc();
  pocketAcc();
  pocketLora();
  pocketPressure();
  digitalWrite(PIN_LED_GREEN, LOW);
  
  delay(800);

  while(PocketQube.button()==LOW);  


}


void blink(int led) {
  digitalWrite(led, LOW);
  delay(250);
  digitalWrite(led, HIGH);  
  delay(250);
}


void pocketAdc() {

  int adc;

  adc = analogRead(ADC_LUX);
  Serial.print("Lux: ");
  Serial.println(adc);
  
  adc = analogRead(ADC_TEMP);
  Serial.print("Temp: ");
  Serial.println(adc);
  
  adc = analogRead(ADC_HUM);
  Serial.print("Hum: ");
  Serial.println(adc);

}

void pocketAcc(){
  LIS3DE lis = LIS3DE();
  lis.enable();
  if(lis.whoAmI() != 0x33){
    Serial.println("LIS3DE not found");
    return;
  }
  Serial.println("LIS3DE found");
  Serial.print("   X: "); Serial.println(lis.getX());
  Serial.print("   Y: "); Serial.println(lis.getY());
  Serial.print("   Z: "); Serial.println(lis.getZ());
/*
  
  Wire.beginTransmission(address);
  Wire.write(0x0f);
  Wire.endTransmission();
  Wire.requestFrom(address, 1);
  int i = Wire.read();  
  
  Serial.println(i);
  */
}

void pocketPressure(){
  Wire.begin();
  Adafruit_MPL115A2 mpl115a2;
  mpl115a2.begin();
  
  float pressureKPA = 0, temperatureC = 0;    

  pressureKPA = mpl115a2.getPressure();  
  Serial.print("Pressure (kPa): "); Serial.print(pressureKPA, 4); Serial.println(" kPa");

  temperatureC = mpl115a2.getTemperature();  
  Serial.print("Temp (*C): "); Serial.print(temperatureC, 1); Serial.println(" *C");
  
}

#include "LoRa/Radio/radio.h"

RadioEvents_t RadioEvents;

#define TX_OUTPUT_POWER        -1  // -1 to 14 dBm
#define LORA_BANDWIDTH          0  // 0: 125 kHz, 1: 250 kHz, 2: 500 kHz, 3: Reserved
#define LORA_DATARATE           7  // 6: 64, 7: 128, 8: 256, 9: 512, 10: 1024, 11: 2048, 12: 4096  chips
#define LORA_CODINGRATE         1  // 1: 4/5, 2: 4/6, 3: 4/7, 4: 4/8
#define LORA_PREAMBLE_LENGTH    8  // Length in symbols (the hardware adds 4 more symbols)
#define LORA_FIX_LENGTH_PAYLOAD 0  // Fixed length packets [0: variable, 1: fixed]
#define LORA_CRC_ENABLED        1  // 0: OFF, 1: ON
#define LORA_FHSS_ENABLED       0  // 0: OFF, 1: ON
#define LORA_NB_SYMB_HOP        4  // Number of symbols between each hop
#define LORA_IQ_INVERSION_ON    0  // Inverts IQ signals [0: not inverted, 1: inverted]
#define LORA_TIMEOUT            2000

#define RF_FREQUENCY            868100000 // CH1

void pocketLoraInit(){
  Serial.println("LoRa Tx Mode");
  Serial.println();

  RadioEvents.TxDone    = OnTxDone;
  RadioEvents.RxDone    = OnRxDone;
  RadioEvents.RxError   = OnRxError;
  RadioEvents.TxTimeout = OnTxTimeout;
  RadioEvents.RxTimeout = OnRxTimeout; 
     
  Radio.Init(&RadioEvents);
  Radio.SetTxConfig( MODEM_LORA, TX_OUTPUT_POWER, 0, LORA_BANDWIDTH,
                       LORA_DATARATE, LORA_CODINGRATE,
                       LORA_PREAMBLE_LENGTH, LORA_FIX_LENGTH_PAYLOAD,
                       LORA_CRC_ENABLED, LORA_FHSS_ENABLED, LORA_NB_SYMB_HOP, 
                       LORA_IQ_INVERSION_ON, LORA_TIMEOUT);
  
  Radio.SetChannel(RF_FREQUENCY);

  if(Radio.GetStatus() != 0){
    Serial.println("LoRa module not found");
    return;
  }
  Serial.println("LoRa module ok");

}

#define BUFFER_SIZE         32       
const uint8_t  SendMsg[]  = "ThisIsATest";
uint16_t       BufferSize = BUFFER_SIZE;
uint8_t        Buffer[BUFFER_SIZE];

void pocketLora(){
  digitalWrite(PIN_LED_GREEN, LOW);
  memcpy(Buffer, SendMsg, sizeof(SendMsg));
  delay(10); 
  Radio.Send( Buffer, BufferSize );
}


void OnTxDone()
{
    Radio.Sleep();
    Serial.println("OnTxDone");
}

void OnRxDone(uint8_t *payload, uint16_t len, int16_t rssi, int8_t snr)
{
    Radio.Sleep( );
    digitalWrite(PIN_LED_RED, LOW);
    Serial.print("RX Length: ");
    Serial.println(len);
    Serial.print("RX RSSI: ");
    Serial.println(rssi);
    Serial.print("RX SNR: ");
    Serial.println(snr);
    Serial.println("");
    
    int i;
    for(i=0; i<len; i++)
    {
      Serial.print(payload[i], HEX);
      Serial.print(" ");
    }
    Serial.println("");
    delay(100);
    digitalWrite(PIN_LED_RED, HIGH);
}

void OnTxTimeout()
{
    Radio.Sleep();
    Serial.println("OnTxTimeout");
}

void OnRxTimeout()
{
    Radio.Sleep();
    Serial.println("OnRxTimeout");
}

void OnRxError()
{
    Radio.Sleep();
    Serial.println("OnRxError");
}



