#include "PocketQube.h"
#include "Wire.h"
#include "LIS3DE.h"
#include "Adafruit_MPL115A2.h"
#include "GPS.h"
#include "Communications.h"
#include "LoRaWAN.h"

const char *app_eui = "xxxxxxxxxxxxxxxx";
const char *app_key = "0A1554BFE1DA1101917C7C6044C255B9";
const char *dev_eui = "xxxxxxxxxxxxxxxx";

GPS    gps = GPS();
LIS3DE acc = LIS3DE();
Adafruit_MPL115A2 mpl115a2 = Adafruit_MPL115A2();

int   x_temperatura;
float temperatura;
float temperatura_media = 0;

int   x_humidade;
float humidade;
float humidade_media = 0;

float acc_z;
float acc_media;

const int periodo_lora = 60;
int   segundos_lora   = 0;
const int periodo_lorawan = 125;
int   segundos_lorawan   = 0;
int   lorawan_next_tx = 0;

int   packet_n   = 0;
char  dados_a_enviar[100];


void setup() {

  Serial.begin(9600);
  PocketQube.init();

  PocketQube.wdtEnable(10000);

  Wire.begin();
  acc.enable();
  gps.begin();
  mpl115a2.begin();

  temperatura_media = 0;
  humidade_media    = 0;
  acc_media         = 0;

  packet_n = 0;
  segundos_lora    = periodo_lora;
  segundos_lorawan = 0;
  //delay(1000);

}

void loop() {

  x_temperatura = analogRead(PA4);
  temperatura = 12.0 - 66.9 + (218.8 * (x_temperatura / 1023.0)); //+12.0 para calibração
  temperatura_media = temperatura_media + temperatura;

  x_humidade = analogRead(PA4);
  humidade = -12.5 + (125 * (x_humidade / 1023.0));
  humidade_media = humidade_media + humidade;

  acc_z = acc.getZ() * 9.8;
  acc_media = acc_media + acc_z;

  float pressureKPA = mpl115a2.getPressure();
  if(pressureKPA > 200 || pressureKPA < 0){
    pressureKPA = 0;
  }
  
  int h = 0, m = 0, s = 0;
  int lat_i = 0, lon_i = 0;
  int lat_f = 0, lon_f = 0;
  int alt = 0;
  bool fix = false;

  if (gps.ready())
  {

    h = gps.getHours();
    m = gps.getMinutes();
    s = gps.getSeconds();
    lat_i = gps.getLatitudeInt();
    lat_f = gps.getLatitudeFrac();
    lon_i = gps.getLongitudeInt();
    lon_f = gps.getLongitudeFrac();
    alt   = gps.getAltitude();
    fix   = gps.isFix();

  }

  if (segundos_lora >= periodo_lora) {
    PocketQube.redOn();
    Communications.LoRa.begin(0);
    Communications.LoRa.setTx(0,14);
    delay(10);
    Communications.LoRa.link();
    delay(10);
    PocketQube.redOff();
    
    segundos_lora = 0;
    packet_n = packet_n + 1;

    
    sprintf(dados_a_enviar, "PQ:%d,%d,%d,%d,%d,%d,%02d%02d%02d,%d.%04d,%d.%04d,%d,%c,%d",
            packet_n,
            (int)(PocketQube.getTemperature() * 10.0),
            (int)(temperatura_media * 10.0 / periodo_lora),
            (int)(humidade_media * 10.0 / periodo_lora),
            (int)(acc_media * 10.0 / periodo_lora),
            (int)(pressureKPA),
            h, m, s,
            lat_i, lat_f, lon_i, lon_f,
            alt,
            fix ? 'F' : '0',
            segundos_lorawan
           );
    Serial.println(dados_a_enviar);
    Communications.LoRa.send(dados_a_enviar);

    temperatura_media = 0;
    humidade_media    = 0;
    acc_media         = 0;

  } else {
    segundos_lora = segundos_lora + 1;
  }



  if(segundos_lorawan <= 0){   
    Serial.println("startlorawan");

    LoRaWAN.begin(EU868);
    LoRaWAN.joinOTAA(app_eui, app_key, dev_eui);
    delay(100);
    if (LoRaWAN.joined() && !LoRaWAN.busy())
    {
          PocketQube.redOn();
          lorawan_next_tx = (Communications.LoRaWAN.getNextTxTime()/1000);
          Serial.println(lorawan_next_tx);
          Communications.LoRaWAN.send(dados_a_enviar);
//
//           LoRaWAN.beginPacket();
//            LoRaWAN.write(0xef);
//            LoRaWAN.write(0xbe);
//            LoRaWAN.write(0xad);
//            LoRaWAN.write(0xde);
//            LoRaWAN.endPacket();
//        
          Serial.print("Send by LoRaWAN: ");
          Serial.print(dados_a_enviar);
          PocketQube.redOff();
      
    }

    //Serial.println("stoplorawan");
    segundos_lorawan = periodo_lorawan;
  }else{
    segundos_lorawan -= 1;
  }

  //if the gps signal is fix -> blink green. otherwise blink blue
  if (fix) {
    PocketQube.greenOn();
    delay(5);
    PocketQube.greenOff();
  } else {
    PocketQube.blueOn();
    delay(5);
    PocketQube.blueOff();
  }
  delay(995 + random(100)); //random to avoid timing colisions

  PocketQube.wdtReset();
}




